@@ -28,10 +28,10 @@
 ## This function is for Matlab compatibility and provides a subset of
 ## the functionality of @code{nonlin_min}.
 ##
-## @var{objf}: objective function. It gets a column vector of real
-## parameters as argument.
+## @var{objf}: objective function. It gets the real parameters as
+## argument.
 ##
-## @var{x0}: real column vector of initial parameters.
+## @var{x0}: real vector or array of initial parameters.
 ##
 ## @var{A}, @var{b}: Inequality constraints of the parameters @code{p}
 ## with @code{A * p - b <= 0}.
@@ -39,7 +39,11 @@
 ## @var{Aeq}, @var{beq}: Equality constraints of the parameters @code{p}
 ## with @code{A * p - b = 0}.
 ##
-## @var{lb}, @var{ub}: Bounds of the parameters @code{p} with @code{lb <= p <= ub}.
+## @var{lb}, @var{ub}: Bounds of the parameters @code{p} with @code{lb
+## <= p <= ub}. Vectors or arrays. If the number of elements is
+## smaller than the number of parameters, as many bounds as present
+## are applied, starting with the first parameter. This is for
+## compatibility with Matlab.
 ##
 ## @var{nonlcon}: Nonlinear constraints. Function returning the
 ## current values of nonlinear inequality constraints (constrained to
@@ -60,23 +64,23 @@
 ## the structure must have the field @code{solver}, set to
 ## @qcode{"fmincon"}.
 ##
-## The returned values are the column vector of final parameters
-## @var{x}, the final value of the objective function @var{fval}, an
-## integer @var{cvg} indicating if and how optimization succeeded or
-## failed, and a structure @var{outp} with additional information,
-## curently with possible fields: @code{iterations}, the number of
-## iterations, @code{funcCount}, the number of objective function
-## calls (indirect calls by gradient function not counted),
-## @code{constrviolation}, the maximum of the constraint
-## violations. The backend may define additional fields. @var{cvg} is
-## greater than zero for success and less than or equal to zero for
-## failure; its possible values depend on the used backend and
-## currently can be @code{0} (maximum number of iterations exceeded),
-## @code{1} (success without further specification of criteria),
-## @code{2} (parameter change less than specified precision in two
-## consecutive iterations), @code{3} (improvement in objective
-## function less than specified), @code{-1} (algorithm aborted by a
-## user function), or @code{-4} (algorithm got stuck).
+## The returned values are the final parameters @var{x}, the final
+## value of the objective function @var{fval}, an integer @var{cvg}
+## indicating if and how optimization succeeded or failed, and a
+## structure @var{outp} with additional information, curently with
+## possible fields: @code{iterations}, the number of iterations,
+## @code{funcCount}, the number of objective function calls (indirect
+## calls by gradient function not counted), @code{constrviolation},
+## the maximum of the constraint violations. The backend may define
+## additional fields. @var{cvg} is greater than zero for success and
+## less than or equal to zero for failure; its possible values depend
+## on the used backend and currently can be @code{0} (maximum number
+## of iterations exceeded), @code{1} (success without further
+## specification of criteria), @code{2} (parameter change less than
+## specified precision in two consecutive iterations), @code{3}
+## (improvement in objective function less than specified), @code{-1}
+## (algorithm aborted by a user function), or @code{-4} (algorithm got
+## stuck).
 ##
 ## @subsubheading Options:
 ##
@@ -192,8 +196,7 @@ function [p, objf, cvg, outp] = fmincon (varargin)
   if (ischar (f.objf = varargin{1}))
     f.objf = str2func (f.objf);
   endif
-
-  pin = varargin{2}(:);
+  f.objf = @ (p, varargin) f.objf (p);
 
   #### processing of settings and consistency checks
 
@@ -227,6 +230,44 @@ function [p, objf, cvg, outp] = fmincon (varargin)
     f.hessian = [];
   endif    
 
+  if (ischar (nonlcon = varargin{9}))
+    nonlcon = str2func (nonlcon);
+  endif
+
+  if (! isempty (nonlcon))
+    nonlcon = @ (p, varargin) nonlcon (p);
+  endif
+
+  ## handle argument shapes in a way compatible to Matlab
+
+  pin = varargin{2};
+  ps_orig = size (pin);
+  pin = pin(:);
+
+  lbound = varargin{7}(:);
+  ubound = varargin{8}(:);
+
+  if (ps_orig(2) > 1)
+
+    f.objf = @ (p, varargin) f.objf (reshape (p, ps_orig), varargin{:});
+
+    if (dfdp_specified)
+      f.dfdp = @ (p, varargin) f.dfdp (reshape (p, ps_orig), varargin{:});
+    endif
+
+    if (! isempty (f.hessian))
+      f.hessian = ...
+      @ (p, varargin) f.hessian (reshape (p, ps_orig), varargin{:});
+    endif
+
+    if (! isempty (nonlcon))
+      nonlcon = @ (p, varargin) nonlcon (reshape (p, ps_orig), varargin{:});
+    endif
+
+  endif
+
+  ## 
+
   if (isempty (o.FinDiffType))
     FinDiffType_onesided = [];
   else
@@ -250,23 +291,20 @@ function [p, objf, cvg, outp] = fmincon (varargin)
   o.user_interaction = o.OutputFcn;
 
   ## process constraints
-  o.lbound = varargin{7};
-  o.ubound = varargin{8};
+  o.lbound = lbound;
+  o.ubound = ubound;
   o.complex_step_derivative_inequc = false;
   o.complex_step_derivative_equc = false;
   o.inequc = o.equc = {};
   if (! isempty (Aineq = varargin{3}))
     bineq = varargin{4};
-    o.inequc = {-Aineq, bineq};
+    o.inequc = {-Aineq.', bineq};
   endif
   if (! isempty (Aeq = varargin{5}))
     beq = varargin{6};
-    o.equc = {-Aeq, beq};
+    o.equc = {-Aeq.', beq};
   endif
-  if (! isempty (nonlcon = varargin{9}))
-    if (ischar (nonlcon))
-      nonlcon = str2func (nonlcon)
-    endif
+  if (! isempty (nonlcon))
     o.inequc{end + 1} = @ (varargin) - out_1_wrapper (nonlcon, varargin{:});
     o.equc{end + 1} = @ (varargin) - out_2_wrapper (nonlcon, varargin{:});
     if (strcmp (o.GradConstr, "on"))
@@ -295,16 +333,17 @@ function [p, objf, cvg, outp] = fmincon (varargin)
   ## collect parameter-related configuration
 
   ## list of parameter related options, 1st column option name, 2nd
-  ## column field name of default vector, 3rd column <expand scalar?>)
+  ## column field name of default vector, 3rd column <expand
+  ## scalar?>, 4th column <expand vector?>
   prel_opts = { ...
-                "lbound", "negInf", false;
-                "ubound", "Inf", false;
-                "max_fract_change", "NA", false;
-                "fract_prec", "NA", false;
-                "diffp", "NA", true;
-                "TypicalX", "NA", true;
-                "FinDiffRelStep", "NA", true;
-                "diff_onesided", "false", true;
+                "lbound", "negInf", false, true;
+                "ubound", "Inf", false, true;
+                "max_fract_change", "NA", false, false;
+                "fract_prec", "NA", false, false;
+                "diffp", "NA", true, false;
+                "TypicalX", "NA", true, false;
+                "FinDiffRelStep", "NA", true, false;
+                "diff_onesided", "false", true, false;
               };
 
   ## use supplied configuration vectors
@@ -416,6 +455,12 @@ function [p, objf, cvg, outp] = fmincon (varargin)
 
   [p, objf, cvg, outp] = backend (f.objf, pin, hook);
 
+  if (ps_orig(2) > 1)
+
+    p = reshape (p, ps_orig);
+
+  endif
+
   if (isargout (4))
 
     constr = f.f_cstr (p, true (numel (f.vc) + o.n_gencstr, 1));
@@ -529,8 +574,8 @@ endfunction
 %! f = @ (p) sumsq (p(1) * exp (p(2) * indep) - [1, 2, 4, 7, 14]);
 %! ## initial values:
 %! init = [.25; .25];
-%! ## linear constraints, A.' * parametervector + B >= 0
-%! A = [1; -1]; B = 0; # p(1) >= p(2);
+%! ## linear constraints, A * parametervector + B >= 0
+%! A = [1, -1]; B = 0; # p(1) >= p(2);
 %!
 %! assert (fmincon (f, init, -A, B), [.6203; .6203], .0001);
 
@@ -540,7 +585,7 @@ endfunction
 %! problem = struct ("objective",
 %!                   @ (p) sumsq (p(1) * exp (p(2) * indep) - [1, 2, 4, 7, 14]),
 %!                   "x0", [.25; .25],
-%!                   "Aineq", [-1; 1],
+%!                   "Aineq", [-1, 1],
 %!                   "bineq", 0,
 %!                   "solver", "fmincon");
 %! assert (fmincon (problem), [.6203; .6203], .0001);
@@ -569,3 +614,30 @@ endfunction
 %! assert (p, [5.2330e+02; -1.5694e+02; -1.9966e-01; 2.9607e+01;
 %!             8.6615e+01; 4.7326e+01; 2.6235e+01; 2.2915e+01;
 %!             3.9470e+01], .01);
+
+
+%!test
+%! ## Same as above with re-ordered parameters to test bound vectors
+%! ## with less elements than parameters.
+%! objf = @ (p) sumsq (p(1:6));
+%! init = [-127; -151; 379; 421; 460; 426; 300; -100; -.1997];
+%! lbound = [0; 0; 0; 0; 0; 0];
+%! inequc = @ (p) vertcat ( ...
+%!                         p(7) + p(8) * exp (-5 * p(9)) + p(1) - 127,
+%!                         p(7) + p(8) * exp (-3 * p(9)) + p(2) - 151,
+%!                         p(7) + p(8) * exp (-p(9)) + p(3) - 379,
+%!                         p(7) + p(8) * exp (p(9)) + p(4) - 421,
+%!                         p(7) + p(8) * exp (3 * p(9)) + p(5) - 460,
+%!                         p(7) + p(8) * exp (5 * p(9)) + p(6) - 426,
+%!                         -p(7) - p(8) * exp (-5 * p(9)) + p(1) + 127,
+%!                         -p(7) - p(8) * exp (-3 * p(9)) + p(2) + 151,
+%!                         -p(7) - p(8) * exp (-p(9)) + p(3) + 379,
+%!                         -p(7) - p(8) * exp (p(9)) + p(4) + 421,
+%!                         -p(7) - p(8) * exp (3 * p(9)) + p(5) + 460,
+%!                         -p(7) - p(8) * exp (5 * p(9)) + p(6) + 426);
+%! [p, objf, cvg, outp] = fmincon (objf, init, [], [], [], [], lbound, [],
+%!                                 @ (p) {- inequc(p), []}{:},
+%!                                 optimset ("Algorithm", "octave_sqp"));
+%! assert (p, [2.9607e+01; 8.6615e+01; 4.7326e+01; 2.6235e+01;
+%!             2.2915e+01; 3.9470e+01; 5.2330e+02; -1.5694e+02;
+%!             -1.9966e-01], .01);
